
import numpy as np
import json
from utils import compute_categories_id, compute_id_good_occ
from scipy.spatial.transform import Rotation
from bbox_2d import bbox_2d
import cv2
from instance_mask import instance
from pose import convert2
from matplotlib import image
from fps_alg import process2
import os


def process_compute(data_name, camera, camera_resized, new_size, Nb_camera, World_begin, Nb_world, list_categories, occ_target, vis):
    transformation = np.matrix([[0.0000000, -1.0000000, 0.0000000],
                                [0.0000000, 0.0000000, -1.0000000],
                                [1.0000000, 0.0000000, 0.0000000]])
    
    scenarios = ["Worlds", "Cameras", "Mix_all"]

    destination_folders_list = {}
    for scenario_loop in scenarios:
        destination_folders_list[scenario_loop] = [f"Generated_{scenario_loop}_Testing", f"Generated_{scenario_loop}_Evaluating", f"Generated_{scenario_loop}_Training", f"Generated_{scenario_loop}_dont_save" ]

    list_count_categories = {}
    for scenario_loop in scenarios :
        list_count_categories[scenario_loop] = {}
        for destination_folder_loop in destination_folders_list[scenario_loop] : # [f"Generated_{scenario}_Testing", f"Generated_{scenario}_Evaluating", f"Generated_{scenario}_Training"] :
            list_count_categories[scenario_loop][destination_folder_loop] = {}

    destination_folders = {}

    for i in range(World_begin, World_begin + Nb_world): # worlds


        if i > 8000 :
            destination_folders["Worlds"] = f"Generated_Worlds_Testing"
        elif i > 6000 :
            destination_folders["Worlds"] = f"Generated_Worlds_Evaluating"
        else :
            destination_folders["Worlds"] = f"Generated_Worlds_Training"
        
        catergories_instance_array_id_to_cat, catergories_instance_array_cat_to_id, catergories_label_to_id = compute_categories_id(data_name, i)
        
        for j in range(1, Nb_camera+1): # cameras
            p = ((i-1)*Nb_camera) + j

            if j > 12 :
                destination_folders["Cameras"] = f"Generated_Cameras_Testing"
            elif j > 9 :
                destination_folders["Cameras"] = f"Generated_Cameras_Evaluating"
            else :
                destination_folders["Cameras"] = f"Generated_Cameras_Training"

            if i > 8000 and j > 12 :
                destination_folders["Mix_all"] = f"Generated_Mix_all_Testing"
            elif i > 6000 and i <=8000 and j > 9 and j >= 12 :
                destination_folders["Mix_all"] = f"Generated_Mix_all_Evaluating"
            elif i <= 6000 and j <= 9 :
                destination_folders["Mix_all"] = f"Generated_Mix_all_Training"
            else : 
                destination_folders["Mix_all"] = f"Generated_Mix_all_dont_save"

            catergories_occ_array = compute_id_good_occ(data_name, p, catergories_instance_array_id_to_cat, catergories_instance_array_cat_to_id, occ_target)

            ### 3D Poses ###
            with open(f'{data_name}/Pose/{p}.json', 'r') as f:
                data_3D_pose = json.load(f)

            ### 2D BBox ###
            with open(f"{data_name}/Bbox_2d/{p}.json", 'r') as f:
                data_Bbox_2d = json.load(f)

            with open(f"{data_name}/Bbox_3d/{p}.json", 'r') as f:
                data_Bbox_3d = json.load(f)

            if len(data_Bbox_2d) != len(data_3D_pose) :
                raise TypeError("size of datas are differents !!")
                
            for scenario_loop in scenarios:
                for destination_folder_loop in destination_folders_list[scenario_loop] :
                    if os.path.isfile(f'{data_name}/{destination_folder_loop}/Count_{p-1}.json'):
                        with open(f'{data_name}/{destination_folder_loop}/Count_{p-1}.json') as f:
                            list_count_categories[scenario_loop][destination_folder_loop] = json.load(f)

            for categories in list_categories:
                if categories in catergories_occ_array.keys():
                    Nb_instance = len(catergories_occ_array[categories])

                    for scenario_loop in scenarios:

                        meta = {}
                        # 
                        if not destination_folders[scenario_loop] in list_count_categories[scenario_loop].keys():
                            list_count_categories[scenario_loop][destination_folders[scenario_loop]] = {}
                        if not categories in list_count_categories[scenario_loop][destination_folders[scenario_loop]].keys():
                            list_count_categories[scenario_loop][destination_folders[scenario_loop]][categories] = {}
                        if f"{Nb_instance}_instances" in list_count_categories[scenario_loop][destination_folders[scenario_loop]][categories].keys() : 
                            list_count_categories[scenario_loop][destination_folders[scenario_loop]][categories][f"{Nb_instance}_instances"] += 1
                        else : 
                            list_count_categories[scenario_loop][destination_folders[scenario_loop]][categories][f"{Nb_instance}_instances"] = 1

                        meta['id_generated'] = list_count_categories[scenario_loop][destination_folders[scenario_loop]][categories][f"{Nb_instance}_instances"]
                        meta['id_original'] = p
                        meta['id_category'] = catergories_label_to_id[categories] 
                        meta['id_instance'] = catergories_occ_array[categories]
                        meta['id_dataset'] = 1
                        meta['world'] = i
                        meta['camera'] = f"grabber_{j}"
                        meta['occlusion'] = occ_target
                        meta['Nb_instance_category'] = Nb_instance

                        if not os.path.isfile(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Meta_Gen/{categories}_{p}.json'):
                            with open(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Meta_Gen/{categories}_{p}.json', mode='w') as f:
                                feeds = {}
                                feeds[meta['id_generated']]=meta
                                f.write(json.dumps(feeds, indent=2))
                        else:
                            with open(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Meta_Gen/{categories}_{p}.json') as feedsjson:
                                feeds = json.load(feedsjson)
                                feeds[meta['id_generated']]=meta
                            with open(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Meta_Gen/{categories}_{p}.json', mode='w') as f:
                                f.write(json.dumps(feeds, indent=4))    

                    if (Nb_instance == 1):
                        for k in range(len(data_3D_pose)):
                            if data_3D_pose[k]['id'] == catergories_occ_array[categories][0]:
                                rpy = data_3D_pose[k]['pose']['rpy']
                                rot = convert2(rpy)
                                R_exp = transformation @ rot
                                R_exp = np.array(R_exp)

                                xyz = data_3D_pose[k]['pose']['xyz']
                                T_exp = transformation @ xyz
                                T_exp = np.array(T_exp)
                                num_arr = np.c_[R_exp, T_exp[0]]
                                np.save(f'{data_name}/Generated/{categories}/Pose_transformed/{p}.npy', num_arr)  # save
                            else:
                                continue

                            if data_Bbox_2d[k]['id'] == catergories_occ_array[categories][0]:
                                bbox = bbox_2d(data_Bbox_2d[k])
                                for scenario_loop in scenarios:
                                    if not destination_folders[scenario_loop] == "dont_save" :
                                        np.savetxt(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Bbox/{p}.txt', np.array(bbox).reshape((1, 4)))  # save
                            else:
                                continue

                            if data_Bbox_3d[k]['id'] == catergories_occ_array[categories][0]:
                                bbox3d_size = data_Bbox_3d[k]['bbox']['size']
                                for scenario_loop in scenarios:
                                    if not destination_folders[scenario_loop] == "dont_save" :
                                        np.savetxt(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/Bbox_3d_Gen/{p}.txt', bbox3d_size)  # save
                            else:
                                continue

                        id = catergories_occ_array[categories][0]
                        img = cv2.imread(f"{data_name}/Instance_Segmentation/{p}.png", cv2.IMREAD_UNCHANGED) # plt.imread(path)

                        instance_img = instance(img, id)
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                cv2.imwrite(f"{data_name}/{destination_folders[scenario_loop]}/{categories}/Instance_Mask/{p}.png", 255*instance_img)
                        instance_img_resized = cv2.resize(instance_img, new_size)
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                cv2.imwrite(f"{data_name}/{destination_folders[scenario_loop]}/{categories}/Instance_Mask_resized/{p}.png", 255*instance_img_resized)

                        img = cv2.imread(f"{data_name}/RGB/{p}.png")
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                cv2.imwrite(f"{data_name}/{destination_folders[scenario_loop]}/{categories}/RGB_Gen/{p}.png", img)
                        img_resized = cv2.resize(img, new_size)
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                cv2.imwrite(f"{data_name}/{destination_folders[scenario_loop]}/{categories}/RGB_resized/{p}.png", img_resized)

                        np.set_printoptions(precision=15)
                        #for scenario_loop in scenarios:
                        pose = np.load(f'{data_name}/Generated/{categories}/Pose_transformed/{p}.npy')
                        R_exp = pose[0:3, 0:3]
                        tVec = pose[0:3, 3]

                        fps_points = np.loadtxt(f'{data_name}/Generated/{categories}/{categories}_fps_3d.txt')
                        center = fps_points.mean(0)
                        fps_points = np.append(fps_points, [center], axis=0)
                        points = process2(fps_points, R_exp, tVec, camera, img, vis)
                        out = [int(catergories_occ_array[categories][0])] #len have to be 1 !!
                        ind = 1
                        for point in points:
                            x = point[0][0] / img.shape[1]
                            y = point[0][1] / img.shape[0]
                            out.append(x)
                            out.append(y)
                            ind += 2
                        
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                np.savetxt(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/FPS/{p}.txt',  np.array(out).reshape(1, len(out)))

                        points_resized = process2(fps_points, R_exp, tVec, camera_resized, img_resized, vis)
                        out_resized = [int(catergories_occ_array[categories][0])] #len have to be 1 !
                        ind_resized = 1
                        for point_resized in points_resized:
                            x_resized = point_resized[0][0] / img_resized.shape[1]
                            y_resized = point_resized[0][1] / img_resized.shape[0]
                            out_resized.append(x_resized)
                            out_resized.append(y_resized)
                            ind_resized += 2
                        for scenario_loop in scenarios:
                            if not destination_folders[scenario_loop] == "dont_save" :
                                np.savetxt(f'{data_name}/{destination_folders[scenario_loop]}/{categories}/FPS_resized/{p}.txt',  np.array(out_resized).reshape(1, len(out_resized)))
    

    for scenario_loop in scenarios:
        for destination_folder_loop in destination_folders_list[scenario_loop] : # [f"Generated_{scenario}_Testing", f"Generated_{scenario}_Evaluating", f"Generated_{scenario}_Training"] :
            with open(f'{data_name}/{destination_folder_loop}/Count_{p}.json', mode='w') as f:
                f.write(json.dumps(list_count_categories[scenario_loop][destination_folder_loop], indent=4))
    with open(f'{data_name}/Count_{p}.json', mode='w') as f:
                f.write(json.dumps(list_count_categories, indent=4))
    print(list_count_categories)
    print(f'{data_name}/{destination_folder_loop}/Count_{p}.json')

